==> theia-0.1.4-py3-none-any.whl: theia/aws/database/neptune.py <==
import pulumi
from pulumi.output import Output
from pulumi_aws import neptune

from typing import Optional, List, Mapping
from pydantic.dataclasses import dataclass, ConfigDict
from theia import ResourceOptions
from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags
from theia.utils.errors import TheiaConfigurationError
from theia.aws.database.parameter_group import ParameterGroup

@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NeptuneSettings:
    allow_major_version_upgrade: Optional[bool] = False
    apply_immediately: Optional[bool] = False
    availability_zones: Optional[list[str]] = None
    backup_retention_period: Optional[int] = 1
    cluster_identifier_prefix: Optional[str] = None
    copy_tags_to_snapshot: Optional[bool] = True
    deletion_protection: Optional[bool] = False
    enable_cloudwatch_logs_exports: Optional[list[str]] = None
    final_snapshot_identifier: Optional[str] = None
    global_cluster_identifier: Optional[str] = None
    iam_database_authentication_enabled: Optional[bool] = False
    iam_roles: Optional[List[str | Output[str]]] = None
    kms_key_arn: Optional[str] = None
    neptune_cluster_parameter_group_name: Optional[str] = None
    neptune_instance_parameter_group_name: Optional[str] = None
    neptune_subnet_group_name: Optional[str] = None
    port: Optional[int] = 8182
    preferred_backup_window: Optional[str] = None
    preferred_maintenance_window: Optional[str] = None
    replication_source_identifier: Optional[str] = None
    serverless_v2_scaling_configuration: Optional[neptune.ClusterServerlessV2ScalingConfigurationArgs] = None
    skip_final_snapshot: Optional[bool] = True
    snapshot_identifier: Optional[str] = None
    storage_encrypted: Optional[bool] = None
    tags: Optional[Mapping[str, str]] = None
    vpc_security_group_ids: Optional[List[str | Output[str]]] = None

    #neptune instance specific arguments
    instance_class: Optional[str] = None
    promotion_tier: Optional[int] = 1
    publicly_accessible: Optional[bool] = False

    #custom inputs
    min_ncu: Optional[float] = None
    max_ncu: Optional[float] = None
    subnet_ids: Optional[List[str]] = None
    neptune_param_family: Optional[str] = None
    cluster_parameter_group_values: Optional[ParameterGroup] = None
    instance_parameter_group_values: Optional[ParameterGroup] = None
    num_of_instances: Optional[int] = 0

    def __post_init__(self):
        if self.instance_class == "db.serverless":
            if self.min_ncu is None or self.max_ncu is None:
                raise TheiaConfigurationError("Both min_cpu and max_cpu must be defined for db.serverless instances.")
    
class Neptune(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        cluster_identifier: str,
        engine: str,
        engine_version: str,
        settings: Optional[NeptuneSettings] = NeptuneSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:neptune", resource_name, None, opts)
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        #check subnet group passed, if not create
        if settings.neptune_subnet_group_name:
            neptune_subnet_group_name = settings.neptune_subnet_group_name
        else:
            neptune_subnet= neptune.SubnetGroup(
                resource_name=f"{resource_name}_subnet_group",
                name=f"{resource_name}_subnet_group",
                subnet_ids=settings.subnet_ids,
                tags=merged_tags
            )
            neptune_subnet_group_name=neptune_subnet.name

        #create cluster parameter group
        if settings.cluster_parameter_group_values:
            cluster_parameters = []
            for param in settings.cluster_parameter_group_values.params:
                cluster_parameters.append(neptune.ClusterParameterGroupParameterArgs(name=param.name, value=param.value))
        else:
            cluster_parameters = [
                neptune.ClusterParameterGroupParameterArgs(
                    name="neptune_enable_audit_log",
                    value=0
                )
            ]
        cluster_param_group=neptune.ClusterParameterGroup(
                resource_name=f"{resource_name}-cluster-parameter-group",
                name=f"{resource_name}-cluster-parameter-group",
                family=settings.neptune_param_family,
                parameters=cluster_parameters,
                tags=merged_tags
            )
        
        #create instance parameter group
        if settings.instance_parameter_group_values:
            instance_parameters = []
            for param in settings.instance_parameter_group_values.params:
                instance_parameters.append(neptune.ParameterGroupParameterArgs(name=param.name, value=param.value))
        else:
            instance_parameters = [
                neptune.ParameterGroupParameterArgs(
                    name="neptune_result_cache",
                    value=0
                )
            ]
        instance_param_group=neptune.ParameterGroup(
            resource_name=f"{resource_name}-instance-parameter-group",
            name=f"{resource_name}-instance-parameter-group",
            family=settings.neptune_param_family,
            parameters=instance_parameters,
            tags=merged_tags
        )

        cluster_args = neptune.ClusterArgs(
            allow_major_version_upgrade=settings.allow_major_version_upgrade,
            apply_immediately= settings.apply_immediately,
            availability_zones = settings.availability_zones,
            backup_retention_period = settings.backup_retention_period,
            cluster_identifier = cluster_identifier,
            cluster_identifier_prefix = settings.cluster_identifier_prefix,
            copy_tags_to_snapshot = settings.copy_tags_to_snapshot,
            deletion_protection = settings.deletion_protection,
            enable_cloudwatch_logs_exports = settings.enable_cloudwatch_logs_exports,
            engine = engine,
            engine_version = engine_version,
            final_snapshot_identifier = f"{resource_name}-final-backup",
            global_cluster_identifier = settings.global_cluster_identifier,
            iam_database_authentication_enabled = settings.iam_database_authentication_enabled,
            iam_roles = settings.iam_roles,
            kms_key_arn = settings.kms_key_arn,
            neptune_cluster_parameter_group_name = cluster_param_group.name,
            neptune_instance_parameter_group_name = instance_param_group.name,
            neptune_subnet_group_name = neptune_subnet_group_name,
            port = settings.port,
            preferred_backup_window = settings.preferred_backup_window,
            preferred_maintenance_window = settings.preferred_maintenance_window,
            replication_source_identifier = settings.replication_source_identifier,
            skip_final_snapshot = settings.skip_final_snapshot,
            snapshot_identifier = settings.snapshot_identifier,
            storage_encrypted = settings.storage_encrypted,
            serverless_v2_scaling_configuration=neptune.ClusterServerlessV2ScalingConfigurationArgs(
                min_capacity=settings.min_ncu,
                max_capacity=settings.max_ncu
            ) if settings.instance_class == "db.serverless" else None,
            tags = merged_tags,
            vpc_security_group_ids = settings.vpc_security_group_ids
        )
        depends = []
        depends.append(cluster_param_group)
        depends.append(instance_param_group)

        if not settings.neptune_subnet_group_name:
            depends.append(neptune_subnet)

        #creates neptune cluster
        self.neptune_cluster = neptune.Cluster(
            resource_name,
            args=cluster_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        instance_endpoints = []
        #creates neptune instances
        for i in range(settings.num_of_instances):
            instance_args=neptune.ClusterInstanceArgs(
                cluster_identifier=self.neptune_cluster.id,
                instance_class=settings.instance_class,
                engine = engine,
                engine_version = engine_version,
                identifier = f"{cluster_identifier}-{i}",
                neptune_parameter_group_name=instance_param_group.name,
                neptune_subnet_group_name=neptune_subnet_group_name,
                tags=merged_tags
            )
            instance = neptune.ClusterInstance(
                resource_name=f"{resource_name}-instance-{i}",
                args=instance_args,
                opts=ResourceOptions(parent=self, depends_on=[self.neptune_cluster])
            )
            instance_endpoints.append(instance.endpoint)
            
        pulumi.export(
            f"neptunecluster---{resource_name}",
            {
                "cluster_arn": self.neptune_cluster.arn,
                "cluster_endpoint": self.neptune_cluster.endpoint,
                "subnet_group_arn": neptune_subnet.arn,
                "cluster_paramater_group_arn": cluster_param_group.arn,
                "instance_paramater_group_arn": instance_param_group.arn,
                "instance_endpoints": instance_endpoints
            },
        )
        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> neptune.Cluster:
        stack_ref = StackRegistry.get(stack_name=stack)
        opensearch = stack_ref.get_output(name=f"opensearch---{resource_name}")
        return opensearch.Domain.get(domain_name=resource_name, arn=opensearch["arn"])

==> theia-0.1.8.31-py3-none-any.whl: theia/aws/database/neptune.py <==
import pulumi
from pulumi.output import Output
from pulumi_aws import neptune

from typing import Optional, List, Mapping
from pydantic.dataclasses import dataclass, ConfigDict
from theia import ResourceOptions
from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags
from theia.utils.errors import TheiaConfigurationError
from theia.aws.database.parameter_group import ParameterGroup

@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NeptuneSettings:
    allow_major_version_upgrade: Optional[bool] = False
    apply_immediately: Optional[bool] = False
    availability_zones: Optional[list[str | Output[str]]] = None
    backup_retention_period: Optional[int] = 1
    cluster_identifier_prefix: Optional[str] = None
    copy_tags_to_snapshot: Optional[bool] = True
    deletion_protection: Optional[bool] = False
    enable_cloudwatch_logs_exports: Optional[list[str]] = None
    final_snapshot_identifier: Optional[str] = None
    global_cluster_identifier: Optional[str] = None
    iam_database_authentication_enabled: Optional[bool] = False
    iam_roles: Optional[List[str | Output[str]]] = None
    kms_key_arn: Optional[str] = None
    neptune_cluster_parameter_group_name: Optional[str] = None
    neptune_instance_parameter_group_name: Optional[str] = None
    neptune_subnet_group_name: Optional[str] = None
    port: Optional[int] = 8182
    preferred_backup_window: Optional[str] = None
    preferred_maintenance_window: Optional[str] = None
    replication_source_identifier: Optional[str] = None
    serverless_v2_scaling_configuration: Optional[neptune.ClusterServerlessV2ScalingConfigurationArgs] = None
    skip_final_snapshot: Optional[bool] = True
    snapshot_identifier: Optional[str] = None
    storage_encrypted: Optional[bool] = None
    tags: Optional[Mapping[str, str]] = None
    vpc_security_group_ids: Optional[List[str | Output[str]]] = None

    #neptune instance specific arguments
    instance_class: Optional[str] = None
    promotion_tier: Optional[int] = 1
    publicly_accessible: Optional[bool] = False

    #custom inputs
    min_ncu: Optional[float] = None
    max_ncu: Optional[float] = None
    subnet_ids: Optional[List[str | Output[str]]] = None
    neptune_param_family: Optional[str] = None
    cluster_parameter_group_values: Optional[ParameterGroup] = None
    instance_parameter_group_values: Optional[ParameterGroup] = None
    num_of_instances: Optional[int] = 0

    def __post_init__(self):
        if self.instance_class == "db.serverless":
            if self.min_ncu is None or self.max_ncu is None:
                raise TheiaConfigurationError("Both min_cpu and max_cpu must be defined for db.serverless instances.")
    
class Neptune(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        cluster_identifier: str,
        engine: str,
        engine_version: str,
        settings: Optional[NeptuneSettings] = NeptuneSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:neptune", resource_name, None, opts)
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        #check subnet group passed, if not create
        if settings.neptune_subnet_group_name:
            neptune_subnet_group_name = settings.neptune_subnet_group_name
        else:
            neptune_subnet= neptune.SubnetGroup(
                resource_name=f"{resource_name}_subnet_group",
                name=f"{resource_name}_subnet_group",
                subnet_ids=settings.subnet_ids,
                tags=merged_tags
            )
            neptune_subnet_group_name=neptune_subnet.name

        #create cluster parameter group
        if settings.cluster_parameter_group_values:
            cluster_parameters = []
            for param in settings.cluster_parameter_group_values.params:
                cluster_parameters.append(neptune.ClusterParameterGroupParameterArgs(name=param.name, value=param.value))
        else:
            cluster_parameters = [
                neptune.ClusterParameterGroupParameterArgs(
                    name="neptune_enable_audit_log",
                    value=0
                )
            ]
        cluster_param_group=neptune.ClusterParameterGroup(
                resource_name=f"{resource_name}-cluster-parameter-group",
                name=f"{resource_name}-cluster-parameter-group",
                family=settings.neptune_param_family,
                parameters=cluster_parameters,
                tags=merged_tags
            )
        
        #create instance parameter group
        if settings.instance_parameter_group_values:
            instance_parameters = []
            for param in settings.instance_parameter_group_values.params:
                instance_parameters.append(neptune.ParameterGroupParameterArgs(name=param.name, value=param.value))
        else:
            instance_parameters = [
                neptune.ParameterGroupParameterArgs(
                    name="neptune_result_cache",
                    value=0
                )
            ]
        instance_param_group=neptune.ParameterGroup(
            resource_name=f"{resource_name}-instance-parameter-group",
            name=f"{resource_name}-instance-parameter-group",
            family=settings.neptune_param_family,
            parameters=instance_parameters,
            tags=merged_tags
        )

        cluster_args = neptune.ClusterArgs(
            allow_major_version_upgrade=settings.allow_major_version_upgrade,
            apply_immediately= settings.apply_immediately,
            availability_zones = settings.availability_zones,
            backup_retention_period = settings.backup_retention_period,
            cluster_identifier = cluster_identifier,
            cluster_identifier_prefix = settings.cluster_identifier_prefix,
            copy_tags_to_snapshot = settings.copy_tags_to_snapshot,
            deletion_protection = settings.deletion_protection,
            enable_cloudwatch_logs_exports = settings.enable_cloudwatch_logs_exports,
            engine = engine,
            engine_version = engine_version,
            final_snapshot_identifier = f"{resource_name}-final-backup",
            global_cluster_identifier = settings.global_cluster_identifier,
            iam_database_authentication_enabled = settings.iam_database_authentication_enabled,
            iam_roles = settings.iam_roles,
            kms_key_arn = settings.kms_key_arn,
            neptune_cluster_parameter_group_name = cluster_param_group.name,
            neptune_instance_parameter_group_name = instance_param_group.name,
            neptune_subnet_group_name = neptune_subnet_group_name,
            port = settings.port,
            preferred_backup_window = settings.preferred_backup_window,
            preferred_maintenance_window = settings.preferred_maintenance_window,
            replication_source_identifier = settings.replication_source_identifier,
            skip_final_snapshot = settings.skip_final_snapshot,
            snapshot_identifier = settings.snapshot_identifier,
            storage_encrypted = settings.storage_encrypted,
            serverless_v2_scaling_configuration=neptune.ClusterServerlessV2ScalingConfigurationArgs(
                min_capacity=settings.min_ncu,
                max_capacity=settings.max_ncu
            ) if settings.instance_class == "db.serverless" else None,
            tags = merged_tags,
            vpc_security_group_ids = settings.vpc_security_group_ids
        )
        depends = []
        depends.append(cluster_param_group)
        depends.append(instance_param_group)

        if not settings.neptune_subnet_group_name:
            depends.append(neptune_subnet)

        #creates neptune cluster
        self.neptune_cluster = neptune.Cluster(
            resource_name,
            args=cluster_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        instance_endpoints = []
        #creates neptune instances
        for i in range(settings.num_of_instances):
            instance_args=neptune.ClusterInstanceArgs(
                cluster_identifier=self.neptune_cluster.id,
                instance_class=settings.instance_class,
                engine = engine,
                engine_version = engine_version,
                identifier = f"{cluster_identifier}-{i}",
                neptune_parameter_group_name=instance_param_group.name,
                neptune_subnet_group_name=neptune_subnet_group_name,
                tags=merged_tags
            )
            instance = neptune.ClusterInstance(
                resource_name=f"{resource_name}-instance-{i}",
                args=instance_args,
                opts=ResourceOptions(parent=self, depends_on=[self.neptune_cluster])
            )
            instance_endpoints.append(instance.endpoint)
            
        pulumi.export(
            f"neptunecluster---{resource_name}",
            {
                "cluster_arn": self.neptune_cluster.arn,
                "cluster_endpoint": self.neptune_cluster.endpoint,
                "subnet_group_arn": neptune_subnet.arn,
                "cluster_paramater_group_arn": cluster_param_group.arn,
                "instance_paramater_group_arn": instance_param_group.arn,
                "instance_endpoints": instance_endpoints
            },
        )
        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> neptune.Cluster:
        stack_ref = StackRegistry.get(stack_name=stack)
        opensearch = stack_ref.get_output(name=f"opensearch---{resource_name}")
        return opensearch.Domain.get(domain_name=resource_name, arn=opensearch["arn"])
